@@ -1,4 +1,7 @@
-use crate::attributes::{parse_attributes, Attributes};
+use crate::{
+    attributes::{parse_attributes, Attributes},
+    with::{make_with_ty, with_inner},
+};
 use proc_macro2::TokenStream;
 use quote::{quote, quote_spanned};
 use syn::{
@@ -8,12 +11,7 @@ use syn::{
 
 pub fn derive(input: DeriveInput) -> Result<TokenStream, Error> {
     let attributes = parse_attributes(&input)?;
-
-    if attributes.copy.is_some() {
-        derive_deserialize_copy_impl(input, &attributes)
-    } else {
-        derive_deserialize_impl(input, &attributes)
-    }
+    derive_deserialize_impl(input, &attributes)
 }
 
 fn derive_deserialize_impl(
@@ -41,6 +39,10 @@ fn derive_deserialize_impl(
         where_clause: input.generics.where_clause.clone(),
     };
 
+    let default_rkyv_path = parse_quote! { ::rkyv };
+    let rkyv_path = attributes.rkyv_path.as_ref().unwrap_or(&default_rkyv_path);
+    let with_ty = make_with_ty(rkyv_path);
+
     let name = &input.ident;
     let (impl_generics, _, _) = impl_input_generics.split_for_impl();
     let (_, ty_generics, where_clause) = input.generics.split_for_impl();
@@ -55,7 +57,7 @@ fn derive_deserialize_impl(
                     .iter()
                     .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                 {
-                    let ty = &field.ty;
+                    let ty = with_ty(field);
                     deserialize_where
                         .predicates
                         .push(parse_quote! { #ty: Archive });
@@ -66,13 +68,23 @@ fn derive_deserialize_impl(
 
                 let deserialize_fields = fields.named.iter().map(|f| {
                     let name = &f.ident;
-                    quote! { #name: self.#name.deserialize(deserializer)? }
+                    let ty = with_ty(f);
+                    let value = with_inner(
+                        f,
+                        parse_quote! {
+                            Deserialize::<#ty, __D>::deserialize(
+                                &self.#name,
+                                deserializer,
+                            )?
+                        },
+                    );
+                    quote! { #name: #value }
                 });
 
                 quote! {
                     impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #deserialize_where {
                         #[inline]
-                        fn deserialize(&self, deserializer: &mut __D) -> core::result::Result<#name #ty_generics, __D::Error> {
+                        fn deserialize(&self, deserializer: &mut __D) -> ::core::result::Result<#name #ty_generics, __D::Error> {
                             Ok(#name {
                                 #(#deserialize_fields,)*
                             })
@@ -87,7 +99,7 @@ fn derive_deserialize_impl(
                     .iter()
                     .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                 {
-                    let ty = &field.ty;
+                    let ty = with_ty(field);
                     deserialize_where
                         .predicates
                         .push(parse_quote! { #ty: Archive });
@@ -96,15 +108,25 @@ fn derive_deserialize_impl(
                         .push(parse_quote! { Archived<#ty>: Deserialize<#ty, __D> });
                 }
 
-                let deserialize_fields = fields.unnamed.iter().enumerate().map(|(i, _)| {
+                let deserialize_fields = fields.unnamed.iter().enumerate().map(|(i, f)| {
                     let index = Index::from(i);
-                    quote! { self.#index.deserialize(deserializer)? }
+                    let ty = with_ty(f);
+                    let value = with_inner(
+                        f,
+                        parse_quote! {
+                            Deserialize::<#ty, __D>::deserialize(
+                                &self.#index,
+                                deserializer,
+                            )?
+                        },
+                    );
+                    quote! { #value }
                 });
 
                 quote! {
                     impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #deserialize_where {
                         #[inline]
-                        fn deserialize(&self, deserializer: &mut __D) -> core::result::Result<#name #ty_generics, __D::Error> {
+                        fn deserialize(&self, deserializer: &mut __D) -> ::core::result::Result<#name #ty_generics, __D::Error> {
                             Ok(#name(
                                 #(#deserialize_fields,)*
                             ))
@@ -115,7 +137,7 @@ fn derive_deserialize_impl(
             Fields::Unit => quote! {
                 impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #where_clause {
                     #[inline]
-                    fn deserialize(&self, _: &mut __D) -> core::result::Result<#name #ty_generics, __D::Error> {
+                    fn deserialize(&self, _: &mut __D) -> ::core::result::Result<#name #ty_generics, __D::Error> {
                         Ok(#name)
                     }
                 }
@@ -131,7 +153,7 @@ fn derive_deserialize_impl(
                             .iter()
                             .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                         {
-                            let ty = &field.ty;
+                            let ty = with_ty(field);
                             deserialize_where
                                 .predicates
                                 .push(parse_quote! { #ty: Archive });
@@ -146,7 +168,7 @@ fn derive_deserialize_impl(
                             .iter()
                             .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                         {
-                            let ty = &field.ty;
+                            let ty = with_ty(field);
                             deserialize_where
                                 .predicates
                                 .push(parse_quote! { #ty: Archive });
@@ -169,9 +191,17 @@ fn derive_deserialize_impl(
                         });
                         let fields = fields.named.iter().map(|f| {
                             let name = &f.ident;
-                            quote! {
-                                #name: #name.deserialize(deserializer)?
-                            }
+                            let ty = with_ty(f);
+                            let value = with_inner(
+                                f,
+                                parse_quote! {
+                                    Deserialize::<#ty, __D>::deserialize(
+                                        #name,
+                                        deserializer,
+                                    )?
+                                },
+                            );
+                            quote! { #name: #value }
                         });
                         quote_spanned! { variant.span() =>
                             Self::#variant { #(#bindings,)* } => #name::#variant { #(#fields,)* }
@@ -184,9 +214,17 @@ fn derive_deserialize_impl(
                         });
                         let fields = fields.unnamed.iter().enumerate().map(|(i, f)| {
                             let binding = Ident::new(&format!("_{}", i), f.span());
-                            quote! {
-                                #binding.deserialize(deserializer)?
-                            }
+                            let ty = with_ty(f);
+                            let value = with_inner(
+                                f,
+                                parse_quote! {
+                                    Deserialize::<#ty, __D>::deserialize(
+                                        #binding,
+                                        deserializer,
+                                    )?
+                                },
+                            );
+                            quote! { #value }
                         });
                         quote_spanned! { variant.span() =>
                             Self::#variant( #(#bindings,)* ) => #name::#variant(#(#fields,)*)
@@ -201,7 +239,7 @@ fn derive_deserialize_impl(
             quote! {
                 impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #deserialize_where {
                     #[inline]
-                    fn deserialize(&self, deserializer: &mut __D) -> core::result::Result<#name #ty_generics, __D::Error> {
+                    fn deserialize(&self, deserializer: &mut __D) -> ::core::result::Result<#name #ty_generics, __D::Error> {
                         Ok(match self {
                             #(#deserialize_variants,)*
                         })
@@ -218,140 +256,9 @@ fn derive_deserialize_impl(
     };
 
     Ok(quote! {
+        #[automatically_derived]
         const _: () = {
-            use rkyv::{Archive, Archived, Deserialize, Fallible};
-            #deserialize_impl
-        };
-    })
-}
-
-fn derive_deserialize_copy_impl(
-    mut input: DeriveInput,
-    attributes: &Attributes,
-) -> Result<TokenStream, Error> {
-    if let Some(ref archived) = attributes.archived {
-        return Err(Error::new_spanned(
-            archived,
-            "archive copy types cannot be named",
-        ));
-    } else if let Some(ref resolver) = attributes.resolver {
-        return Err(Error::new_spanned(
-            resolver,
-            "archive copy resolvers cannot be named",
-        ));
-    };
-
-    input.generics.make_where_clause();
-
-    let mut impl_input_params = Punctuated::default();
-    impl_input_params.push(parse_quote! { __D: Fallible + ?Sized });
-    for param in input.generics.params.iter() {
-        impl_input_params.push(param.clone());
-    }
-    let impl_input_generics = Generics {
-        lt_token: Some(Default::default()),
-        params: impl_input_params,
-        gt_token: Some(Default::default()),
-        where_clause: input.generics.where_clause.clone(),
-    };
-
-    let name = &input.ident;
-    let (impl_generics, _, _) = impl_input_generics.split_for_impl();
-    let (_, ty_generics, where_clause) = input.generics.split_for_impl();
-    let where_clause = where_clause.unwrap();
-
-    let deserialize_impl = match input.data {
-        Data::Struct(ref data) => {
-            let mut deserialize_where = where_clause.clone();
-            match data.fields {
-                Fields::Named(ref fields) => {
-                    for field in fields
-                        .named
-                        .iter()
-                        .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
-                    {
-                        let ty = &field.ty;
-                        deserialize_where
-                            .predicates
-                            .push(parse_quote! { #ty: ArchiveCopy });
-                    }
-                }
-                Fields::Unnamed(ref fields) => {
-                    for field in fields
-                        .unnamed
-                        .iter()
-                        .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
-                    {
-                        let ty = &field.ty;
-                        deserialize_where
-                            .predicates
-                            .push(parse_quote! { #ty: ArchiveCopy });
-                    }
-                }
-                Fields::Unit => (),
-            }
-
-            quote! {
-                impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #deserialize_where {
-                    #[inline]
-                    fn deserialize(&self, _: &mut __D) -> core::result::Result<Self, __D::Error> {
-                        Ok(*self)
-                    }
-                }
-            }
-        }
-        Data::Enum(ref data) => {
-            let mut deserialize_where = where_clause.clone();
-            for variant in data.variants.iter() {
-                match variant.fields {
-                    Fields::Named(ref fields) => {
-                        for field in fields
-                            .named
-                            .iter()
-                            .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
-                        {
-                            let ty = &field.ty;
-                            deserialize_where
-                                .predicates
-                                .push(parse_quote! { #ty: ArchiveCopy });
-                        }
-                    }
-                    Fields::Unnamed(ref fields) => {
-                        for field in fields
-                            .unnamed
-                            .iter()
-                            .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
-                        {
-                            let ty = &field.ty;
-                            deserialize_where
-                                .predicates
-                                .push(parse_quote! { #ty: ArchiveCopy });
-                        }
-                    }
-                    Fields::Unit => (),
-                }
-            }
-
-            quote! {
-                impl #impl_generics Deserialize<#name #ty_generics, __D> for Archived<#name #ty_generics> #deserialize_where {
-                    #[inline]
-                    fn deserialize(&self, _: &mut __D) -> core::result::Result<Self, __D::Error> {
-                        Ok(*self)
-                    }
-                }
-            }
-        }
-        Data::Union(_) => {
-            return Err(Error::new_spanned(
-                input,
-                "Deserialize cannot be derived for unions",
-            ));
-        }
-    };
-
-    Ok(quote! {
-        const _: () = {
-            use rkyv::{ArchiveCopy, Archived, Deserialize, Fallible};
+            use #rkyv_path::{Archive, Archived, Deserialize, Fallible};
             #deserialize_impl
         };
     })
