@@ -1,4 +1,7 @@
-use crate::attributes::{parse_attributes, Attributes};
+use crate::{
+    attributes::{parse_attributes, Attributes},
+    with::{make_with_cast, make_with_ty},
+};
 use proc_macro2::TokenStream;
 use quote::{quote, quote_spanned};
 use syn::{
@@ -8,12 +11,7 @@ use syn::{
 
 pub fn derive(input: DeriveInput) -> Result<TokenStream, Error> {
     let attributes = parse_attributes(&input)?;
-
-    if attributes.copy.is_some() {
-        derive_serialize_copy_impl(input, &attributes)
-    } else {
-        derive_serialize_impl(input, &attributes)
-    }
+    derive_serialize_impl(input, &attributes)
 }
 
 fn derive_serialize_impl(
@@ -41,6 +39,11 @@ fn derive_serialize_impl(
         where_clause: input.generics.where_clause.clone(),
     };
 
+    let default_rkyv_path = parse_quote! { ::rkyv };
+    let rkyv_path = attributes.rkyv_path.as_ref().unwrap_or(&default_rkyv_path);
+    let with_ty = make_with_ty(rkyv_path);
+    let with_cast = make_with_cast(rkyv_path);
+
     let name = &input.ident;
     let (impl_generics, _, _) = impl_input_generics.split_for_impl();
     let (_, ty_generics, where_clause) = input.generics.split_for_impl();
@@ -60,7 +63,7 @@ fn derive_serialize_impl(
                     .iter()
                     .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                 {
-                    let ty = &field.ty;
+                    let ty = with_ty(field);
                     serialize_where
                         .predicates
                         .push(parse_quote! { #ty: Serialize<__S> });
@@ -68,13 +71,14 @@ fn derive_serialize_impl(
 
                 let resolver_values = fields.named.iter().map(|f| {
                     let name = &f.ident;
-                    quote_spanned! { f.span() => #name: Serialize::<__S>::serialize(&self.#name, serializer)? }
+                    let field = with_cast(f, parse_quote! { &self.#name });
+                    quote_spanned! { f.span() => #name: Serialize::<__S>::serialize(#field, serializer)? }
                 });
 
                 quote! {
                     impl #impl_generics Serialize<__S> for #name #ty_generics #serialize_where {
                         #[inline]
-                        fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
+                        fn serialize(&self, serializer: &mut __S) -> ::core::result::Result<Self::Resolver, __S::Error> {
                             Ok(#resolver {
                                 #(#resolver_values,)*
                             })
@@ -89,7 +93,7 @@ fn derive_serialize_impl(
                     .iter()
                     .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                 {
-                    let ty = &field.ty;
+                    let ty = with_ty(field);
                     serialize_where
                         .predicates
                         .push(parse_quote! { #ty: Serialize<__S> });
@@ -97,13 +101,14 @@ fn derive_serialize_impl(
 
                 let resolver_values = fields.unnamed.iter().enumerate().map(|(i, f)| {
                     let index = Index::from(i);
-                    quote_spanned! { f.span() => Serialize::<__S>::serialize(&self.#index, serializer)? }
+                    let field = with_cast(f, parse_quote! { &self.#index });
+                    quote_spanned! { f.span() => Serialize::<__S>::serialize(#field, serializer)? }
                 });
 
                 quote! {
                     impl #impl_generics Serialize<__S> for #name #ty_generics #serialize_where {
                         #[inline]
-                        fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
+                        fn serialize(&self, serializer: &mut __S) -> ::core::result::Result<Self::Resolver, __S::Error> {
                             Ok(#resolver(
                                 #(#resolver_values,)*
                             ))
@@ -115,7 +120,7 @@ fn derive_serialize_impl(
                 quote! {
                     impl #impl_generics Serialize<__S> for #name #ty_generics #where_clause {
                         #[inline]
-                        fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
+                        fn serialize(&self, serializer: &mut __S) -> ::core::result::Result<Self::Resolver, __S::Error> {
                             Ok(#resolver)
                         }
                     }
@@ -132,7 +137,7 @@ fn derive_serialize_impl(
                             .iter()
                             .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                         {
-                            let ty = &field.ty;
+                            let ty = with_ty(field);
                             serialize_where
                                 .predicates
                                 .push(parse_quote! { #ty: Serialize<__S> });
@@ -144,7 +149,7 @@ fn derive_serialize_impl(
                             .iter()
                             .filter(|f| !f.attrs.iter().any(|a| a.path.is_ident("omit_bounds")))
                         {
-                            let ty = &field.ty;
+                            let ty = with_ty(field);
                             serialize_where
                                 .predicates
                                 .push(parse_quote! { #ty: Serialize<__S> });
@@ -164,8 +169,9 @@ fn derive_serialize_impl(
                         });
                         let fields = fields.named.iter().map(|f| {
                             let name = &f.ident;
+                            let field = with_cast(f, parse_quote! { #name });
                             quote! {
-                                #name: Serialize::<__S>::serialize(#name, serializer)?
+                                #name: Serialize::<__S>::serialize(#field, serializer)?
                             }
                         });
                         quote_spanned! { variant.span() =>
@@ -181,8 +187,9 @@ fn derive_serialize_impl(
                         });
                         let fields = fields.unnamed.iter().enumerate().map(|(i, f)| {
                             let binding = Ident::new(&format!("_{}", i), f.span());
+                            let field = with_cast(f, parse_quote! { #binding });
                             quote! {
-                                Serialize::<__S>::serialize(#binding, serializer)?
+                                Serialize::<__S>::serialize(#field, serializer)?
                             }
                         });
                         quote_spanned! { variant.span() =>
@@ -198,7 +205,7 @@ fn derive_serialize_impl(
             quote! {
                 impl #impl_generics Serialize<__S> for #name #ty_generics #serialize_where {
                     #[inline]
-                    fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
+                    fn serialize(&self, serializer: &mut __S) -> ::core::result::Result<Self::Resolver, __S::Error> {
                         Ok(match self {
                             #(#serialize_arms,)*
                         })
@@ -215,143 +222,10 @@ fn derive_serialize_impl(
     };
 
     Ok(quote! {
+        #[automatically_derived]
         const _: () = {
-            use rkyv::{Archive, Fallible, Serialize};
+            use #rkyv_path::{Archive, Fallible, Serialize};
             #serialize_impl
         };
     })
 }
-
-fn derive_serialize_copy_impl(
-    mut input: DeriveInput,
-    attributes: &Attributes,
-) -> Result<TokenStream, Error> {
-    if let Some(ref archived) = attributes.archived {
-        return Err(Error::new_spanned(
-            archived,
-            "archive copy types cannot be named",
-        ));
-    } else if let Some(ref resolver) = attributes.resolver {
-        return Err(Error::new_spanned(
-            resolver,
-            "archive copy resolvers cannot be named",
-        ));
-    };
-
-    input.generics.make_where_clause();
-
-    let mut impl_input_params = Punctuated::default();
-    impl_input_params.push(parse_quote! { __S: Fallible + ?Sized });
-    for param in input.generics.params.iter() {
-        impl_input_params.push(param.clone());
-    }
-    let impl_input_generics = Generics {
-        lt_token: Some(Default::default()),
-        params: impl_input_params,
-        gt_token: Some(Default::default()),
-        where_clause: input.generics.where_clause.clone(),
-    };
-
-    let name = &input.ident;
-    let (impl_generics, _, _) = impl_input_generics.split_for_impl();
-    let (_, ty_generics, where_clause) = input.generics.split_for_impl();
-    let where_clause = where_clause.unwrap();
-
-    let serialize_copy_impl = match input.data {
-        Data::Struct(ref data) => {
-            let mut copy_where = where_clause.clone();
-            match data.fields {
-                Fields::Named(ref fields) => {
-                    for field in fields.named.iter() {
-                        let ty = &field.ty;
-                        copy_where
-                            .predicates
-                            .push(parse_quote! { #ty: ArchiveCopy });
-                    }
-                }
-                Fields::Unnamed(ref fields) => {
-                    for field in fields.unnamed.iter() {
-                        let ty = &field.ty;
-                        copy_where
-                            .predicates
-                            .push(parse_quote! { #ty: ArchiveCopy });
-                    }
-                }
-                Fields::Unit => (),
-            };
-
-            quote! {
-                impl #impl_generics Serialize<__S> for #name #ty_generics #copy_where {
-                    #[inline]
-                    fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
-                        Ok(())
-                    }
-                }
-            }
-        }
-        Data::Enum(ref data) => {
-            if let Some(ref path) = attributes
-                .repr
-                .rust
-                .as_ref()
-                .or_else(|| attributes.repr.transparent.as_ref())
-                .or_else(|| attributes.repr.packed.as_ref())
-            {
-                return Err(Error::new_spanned(
-                    path,
-                    "archive copy enums must be repr(C) or repr(Int)",
-                ));
-            }
-
-            if attributes.repr.c.is_none() && attributes.repr.int.is_none() {
-                return Err(Error::new_spanned(
-                    input,
-                    "archive copy enums must be repr(C) or repr(Int)",
-                ));
-            }
-
-            let mut copy_where = where_clause.clone();
-            for variant in data.variants.iter() {
-                match variant.fields {
-                    Fields::Named(ref fields) => {
-                        for field in fields.named.iter() {
-                            let ty = &field.ty;
-                            copy_where
-                                .predicates
-                                .push(parse_quote! { #ty: ArchiveCopy });
-                        }
-                    }
-                    Fields::Unnamed(ref fields) => {
-                        for field in fields.unnamed.iter() {
-                            let ty = &field.ty;
-                            copy_where
-                                .predicates
-                                .push(parse_quote! { #ty: ArchiveCopy });
-                        }
-                    }
-                    Fields::Unit => (),
-                }
-            }
-
-            quote! {
-                impl #impl_generics Serialize<__S> for #name #ty_generics #copy_where {
-                    #[inline]
-                    fn serialize(&self, serializer: &mut __S) -> core::result::Result<Self::Resolver, __S::Error> {
-                        Ok(())
-                    }
-                }
-            }
-        }
-        Data::Union(_) => {
-            Error::new(input.span(), "Serialize cannot be derived for unions").to_compile_error()
-        }
-    };
-
-    Ok(quote! {
-        const _: () = {
-            use rkyv::{Archive, ArchiveCopy, Fallible, Serialize};
-
-            #serialize_copy_impl
-        };
-    })
-}
